import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_bottom_sheet_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

import 'package:provider/provider.dart';

class SeniorBottomSheet {
  /// Adds a bottom sheet component to the application.
  ///
  /// The [context], [height], [content] and [hasCloseButton] parameters are required.
  /// The [context] parameter expects to receive the current application context.
  /// The [height] parameter specifies the height of the bottom sheet.
  /// The [content] parameter specifies the list of internal elements that will be presented in the bottom sheet.
  /// The [hasCloseButton] parameter specifies whether the bottom sheet will have a button to close it.
  /// The [title] parameter sets a title for the bottom sheet.
  /// The [titleTextStyle] parameter sets the bottom sheet title text style.
  /// The [onTapCloseButton] parameter specifies a function that will be executed whenever the close button is tapped.
  /// The [style] parameter contains the component's style settings. 
  /// The [isDismissible] parameter specifies whether the bottom sheet will be dismissed when user taps on the scrim.
  /// The [enableDrag] parameter specifies whether the bottom sheet can be dragged up and down and dismissed by swiping
  /// downwards.
  static Future<dynamic> showBottomSheet({
    required BuildContext context,
    required double height,
    required List<Widget> content,
    required bool hasCloseButton,
    String? title,
    TextStyle? titleTextStyle,
    VoidCallback? onTapCloseButton,
    SeniorBottomSheetStyle? style,
    bool isDismissible = false,
    bool enableDrag = false,
  }) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context, listen: false);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return showModalBottomSheet(
      isDismissible: isDismissible,
      enableDrag: enableDrag,
      isScrollControlled: true,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(
            SeniorRadius.huge,
          ),
          topRight: Radius.circular(
            SeniorRadius.huge,
          ),
        ),
      ),
      backgroundColor: style?.backgroundColor ??
          theme?.bottomSheetTheme?.style?.backgroundColor ??
          defaultTheme.bottomSheetTheme?.style?.backgroundColor ??
          SeniorColors.pureWhite,
      context: context,
      constraints: BoxConstraints(
        maxHeight: height,
      ),
      builder: (BuildContext context) {
        return Column(
          children: [
            SizedBox(height: SeniorSpacing.medium),
            Padding(
              padding: const EdgeInsets.symmetric(
                horizontal: SeniorSpacing.normal,
              ),
              child: Row(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Expanded(
                    child: title != null
                        ? Text(
                            title,
                            style: SeniorTypography.title(
                                  color: style?.titleColor ??
                                      theme?.bottomSheetTheme?.style?.titleColor ??
                                      defaultTheme.bottomSheetTheme?.style?.titleColor ??
                                      SeniorColors.neutralColor900,
                                ),
                          )
                        : SizedBox.shrink(),
                  ),
                  SizedBox(width: SeniorSpacing.normal),
                  hasCloseButton
                      ? TextButton(
                          child: FaIcon(
                            FontAwesomeIcons.times,
                            size: SeniorTypographyIconSize.CTA,
                            color: style?.closeButtonColor ??
                                theme?.bottomSheetTheme?.style?.closeButtonColor ??
                                defaultTheme.bottomSheetTheme?.style?.closeButtonColor ??
                                SeniorColors.primaryColor300,
                          ),
                          onPressed: onTapCloseButton,
                          style: TextButton.styleFrom(
                            shape: CircleBorder(),
                          ),
                        )
                      : SizedBox.shrink()
                ],
              ),
            ),
            ...content,
          ],
        );
      },
    );
  }
}
