import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_menu_list_item_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

class SeniorMenuItemList extends StatelessWidget {
  /// Creates a menu list item according to SDS.
  ///
  /// The [title] parameter is required.
  const SeniorMenuItemList({
    Key? key,
    this.iconData,
    required this.title,
    this.onTap,
    this.subtitle,
    this.enabled = true,
    this.leftPadding = 0,
    this.rightPadding = 0,
    this.style,
    this.titleMaxLines = 1,
  }) : super(key: key);

  /// An icon for the menu item.
  final IconData? iconData;

  /// The title of the menu item.
  final String title;

  /// The subtitle of the menu item.
  final String? subtitle;

  /// Function performed when the menu item is tapped.
  final VoidCallback? onTap;

  /// Defines whether the menu item will be enabled.
  ///
  /// The default value is true.
  final bool enabled;

  /// The value for a padding to the left of the menu item.
  ///
  /// The default value is 0.
  final double leftPadding;  

  /// The value for a padding to the right of the menu item.
  ///
  /// The default value is 0.
  final double rightPadding;

  /// The component's style definitions.
  /// Allows you to configure:
  /// [SeniorMenuListItemStyle.iconColor] the color of the item's icon.
  /// [SeniorMenuListItemStyle.disabledPushIconColor] the color of the item's action icon.
  /// [SeniorMenuListItemStyle.pushIconColor] the color of the item's action icon when disabled.
  /// [SeniorMenuListItemStyle.subtitleColor] the item's subtitle color.
  /// [SeniorMenuListItemStyle.titleColor] the color of the item's title.
  final SeniorMenuListItemStyle? style;

  /// Defines the maximum lines for the menu item title.
  /// 
  /// The default value is 1.
  final int titleMaxLines;

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return Material(
      color: Colors.transparent,
      child: InkWell(
        onTap: enabled ? onTap : null,
        child: Container(
          padding: EdgeInsets.only(
            top: SeniorSpacing.small,
            bottom: SeniorSpacing.small,
            left: leftPadding,
            right: rightPadding,
          ),
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              iconData != null
                  ? Padding(
                      padding: const EdgeInsets.only(
                        right: SeniorSpacing.small,
                      ),
                      child: Icon(
                        iconData,
                        size: 20,
                        color: style?.iconColor ??
                            theme?.menuListItemTheme?.style?.iconColor ??
                            defaultTheme.menuListItemTheme?.style?.iconColor ??
                            SeniorColors.primaryColor500,
                      ),
                    )
                  : SizedBox.shrink(),
              Expanded(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: [
                    Text(
                      title,
                      maxLines: titleMaxLines,
                      overflow: TextOverflow.ellipsis,
                      textAlign: TextAlign.left,
                      style: SeniorTypography.body(
                        color: style?.titleColor ??
                            theme?.menuListItemTheme?.style?.titleColor ??
                            defaultTheme.menuListItemTheme?.style?.titleColor ??
                            SeniorColors.primaryColor900,
                      ),
                    ),
                    subtitle == null
                        ? SizedBox.shrink()
                        : Text(
                            subtitle!,
                            textAlign: TextAlign.left,
                            style: SeniorTypography.small(
                              color: style?.subtitleColor ??
                                  theme?.menuListItemTheme?.style?.subtitleColor ??
                                  defaultTheme.menuListItemTheme?.style?.subtitleColor ??
                                  SeniorColors.neutralColor500,
                            ),
                          ),
                  ],
                ),
              ),
              Offstage(
                offstage: onTap == null,
                child: Align(
                  alignment: Alignment.center,
                  child: Icon(
                    FontAwesomeIcons.angleRight,
                    color: enabled
                        ? style?.pushIconColor ??
                            theme?.menuListItemTheme?.style?.pushIconColor ??
                            defaultTheme.menuListItemTheme?.style?.pushIconColor ??
                            SeniorColors.primaryColor500
                        : style?.disabledPushIconColor ??
                            theme?.menuListItemTheme?.style?.disabledPushIconColor ??
                            defaultTheme.menuListItemTheme?.style?.disabledPushIconColor ??
                            SeniorColors.neutralColor900,
                    size: 20,
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
