import 'package:flutter/material.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_progress_bar_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

class SeniorProgressBar extends StatelessWidget {
  ///  Creates the SDS ProgressBar component.
  ///
  /// The [currentStep] and [totalSteps] parameters are required.
  const SeniorProgressBar({
    Key? key,
    required this.currentStep,
    this.showCurrentStep = false,
    this.style,
    this.subtitle,
    this.title,
    required this.totalSteps,
  })  : assert(currentStep <= totalSteps),
        assert(title != null && subtitle != null || title == null && subtitle == null),
        super(key: key);

  /// The current step of the scrollbar.
  /// The value of this parameter must be less than the value of the [totalSteps] parameter.
  final int currentStep;

  /// The total scrollbar steps.
  /// The value of this parameter must be greater than the value of the [currentStep] parameter.
  final int totalSteps;

  /// Defines whether to display a label that shows the current step.
  ///
  /// The default value is false.
  final bool showCurrentStep;

  /// The component's style definitions.
  /// Allows you to configure:
  /// [SeniorProgressBarStyle.backgroundColor] the background color of the scrollbar.
  /// [SeniorProgressBarStyle.color] the color of the current scrollbar level.
  final SeniorProgressBarStyle? style;

  /// The subtitle of the progress bar.
  final String? subtitle;

  /// The title of the progress bar.
  final String? title;

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return Padding(
      padding: const EdgeInsets.all(SeniorSpacing.normal),
      child: Column(
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              title != null && subtitle != null
                  ? Row(
                      children: [
                        Text(
                          '$title: ',
                          style: SeniorTypography.text(
                            color: SeniorColors.secondaryColor900,
                          ),
                        ),
                        Text(
                          subtitle!,
                          style: SeniorTypography.body(
                            color: SeniorColors.neutralColor600,
                          ),
                        ),
                      ],
                    )
                  : SizedBox.shrink(),
              showCurrentStep
                  ? Text(
                      '$currentStep/$totalSteps',
                      style: SeniorTypography.body(
                        color: SeniorColors.secondaryColor900,
                      ),
                    )
                  : SizedBox.shrink(),
            ],
          ),
          SizedBox(height: SeniorSpacing.xsmall),
          LinearProgressIndicator(
            color: style?.color ??
                theme?.progressBarTheme?.style?.color ??
                defaultTheme.progressBarTheme?.style?.color ??
                SeniorColors.primaryColor400,
            backgroundColor: style?.backgroundColor ??
                theme?.progressBarTheme?.style?.backgroundColor ??
                defaultTheme.progressBarTheme?.style?.backgroundColor ??
                SeniorColors.neutralColor400,
            minHeight: 8.0,
            value: currentStep / totalSteps,
          ),
        ],
      ),
    );
  }
}
