import 'package:flutter/material.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_radio_button_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

class SeniorRadioButton<T> extends StatelessWidget {
  /// Creates the Radio Button component of the SDS.
  ///
  /// The [groupValue], [onChanged], [title] and [value] parameters are required.
  const SeniorRadioButton({
    Key? key,
    this.disabled = false,
    required this.groupValue,
    required this.onChanged,
    this.style,
    this.toggleable = false,
    required this.title,
    required this.value,
  }) : super(key: key);

  /// Defines if the radio button will be disabled.
  final bool disabled;

  /// Defines the value that will be assigned to the internal [Radio] groupValue property.
  /// The currently selected value for a group of radio buttons.
  final T groupValue;

  /// Defines the value that will be assigned to the internal [Radio] onChanged property.
  /// Called when the user selects this radio button.
  final Function(T?) onChanged;

  /// The component's style definitions.
  ///
  /// Allows you to configure:
  /// [SeniorRadioButtonStyle.disabledFillColor] the fill color of the radio button when disabled.
  /// [SeniorRadioButtonStyle.disabledTitleColor] the radio button title color when disabled.
  /// [SeniorRadioButtonStyle.fillColor] the fill color of the radio button.
  /// [SeniorRadioButtonStyle.titleColor] the radio button title color.
  final SeniorRadioButtonStyle? style;

  /// The title of the radio button.
  final String title;

  /// Defines the value that will be assigned to the internal [Radio] toggleable property.
  /// Set to true if this radio button is allowed to be returned to an indeterminate state by selecting it again when selected.
  final bool toggleable;

  /// Defines the value that will be assigned to the internal [Radio] value property.
  /// The value represented by this radio button.
  final T value;

  Container _buildRadioButton(Color fillColor) {
    return Container(
      height: 16.0,
      width: 16.0,
      margin: EdgeInsets.only(right: SeniorSpacing.xsmall),
      child: Radio<T?>(
        key: key,
        groupValue: groupValue,
        value: value,
        onChanged: !disabled ? onChanged : null,
        fillColor: MaterialStateColor.resolveWith(
          (state) => fillColor,
        ),
        toggleable: toggleable,
      ),
    );
  }

  Flexible _buildTitle(Color titleColor) {
    return Flexible(
      child: Text(
        title,
        style: SeniorTypography.text(
          color: titleColor,
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme =
        SeniorThemeProvider.getDefaultTheme(context);

    final Color titleColor = disabled
        ? style?.disabledTitleColor ??
            theme?.radioButtonTheme?.style?.disabledTitleColor ??
            defaultTheme.radioButtonTheme?.style?.disabledTitleColor ??
            SeniorColors.secondaryColor300
        : style?.titleColor ??
            theme?.radioButtonTheme?.style?.titleColor ??
            defaultTheme.radioButtonTheme?.style?.titleColor ??
            SeniorColors.neutralColor900;

    final Color fillColor = disabled
        ? style?.disabledFillColor ??
            theme?.radioButtonTheme?.style?.disabledFillColor ??
            defaultTheme.radioButtonTheme?.style?.disabledFillColor ??
            SeniorColors.secondaryColor300
        : style?.fillColor ??
            theme?.radioButtonTheme?.style?.fillColor ??
            defaultTheme.radioButtonTheme?.style?.fillColor ??
            SeniorColors.primaryColor;

    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 0, vertical: 8.0),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          _buildRadioButton(fillColor),
          _buildTitle(titleColor),
        ],
      ),
    );
  }
}
