import 'package:flutter/cupertino.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_switch_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

/// The possible SeniorSwitch title positions.
/// It can be [SeniorSwitchTitlePosition.left] and [SeniorSwitchTitlePosition.right].
enum SeniorSwitchTitlePosition {
  left,
  right,
}

class SeniorSwitch extends StatelessWidget {
  /// Creates a Switch component according to SDS.
  ///
  /// The [onChanged] and [value] parameters are required.
  const SeniorSwitch({
    Key? key,
    this.disabled = false,
    this.fullLenght = false,
    this.titlePosition = SeniorSwitchTitlePosition.left,
    required this.onChanged,
    this.style,
    this.title,
    required this.value,
  }) : super(key: key);

  /// Whether the switch will be disabled.
  ///
  /// The default value is false.
  final bool disabled;

  /// Whether the switch will fill all available space.
  ///
  /// The default value is false.
  final bool fullLenght;

  /// The position of the Switch title.
  ///
  /// The default value is [SeniorSwitchTitlePosition.left].
  final SeniorSwitchTitlePosition titlePosition;

  /// Function executed when the switch value is changed.
  ///
  /// It receives the current value of the switch as a parameter.
  final Function(bool?) onChanged;

  /// The component's style definitions.
  ///
  /// Allows you to configure:
  /// [SeniorSwitchStyle.activeColor] the color to use when this switch is on.
  /// [SeniorSwitchStyle.disabledTextColor] the color of the switch's title text when it is disabled.
  /// [SeniorSwitchStyle.textColor] the switch title text color.
  /// [SeniorSwitchStyle.trackColor] the color of this Switch's track.
  final SeniorSwitchStyle? style;

  /// The title of the switch.
  final String? title;

  /// The value of the switch. Defines whether the switch will be checked or unchecked.
  final bool value;

  Widget _buildSwitch(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return Transform.scale(
      scale: 0.7,
      child: CupertinoSwitch(
        activeColor: style?.activeColor ??
            theme?.switchTheme?.style?.activeColor ??
            defaultTheme.switchTheme?.style?.activeColor ??
            SeniorColors.primaryColor,
        onChanged: !disabled ? onChanged : null,
        trackColor: style?.trackColor ??
            theme?.switchTheme?.style?.trackColor ??
            defaultTheme.switchTheme?.style?.trackColor ??
            SeniorColors.secondaryColor600,
        value: value,
      ),
    );
  }

  Widget _buildTitle(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return title != null
        ? Flexible(
            child: Text(
              title!,
              style: SeniorTypography.text(
                color: disabled
                    ? style?.disabledTextColor ??
                        theme?.switchTheme?.style?.disabledTextColor ??
                        defaultTheme.switchTheme?.style?.disabledTextColor ??
                        SeniorColors.secondaryColor300
                    : style?.textColor ??
                        theme?.switchTheme?.style?.textColor ??
                        defaultTheme.switchTheme?.style?.textColor ??
                        SeniorColors.neutralColor900,
              ),
            ),
          )
        : SizedBox.shrink();
  }

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme =
        SeniorThemeProvider.getDefaultTheme(context);

    if(title == null) {
      return _buildSwitch(theme, defaultTheme);
    }

    return Row(
      mainAxisAlignment:
          fullLenght ? MainAxisAlignment.spaceBetween : MainAxisAlignment.start,
      children: titlePosition == SeniorSwitchTitlePosition.left
          ? [
              _buildTitle(theme, defaultTheme),
              _buildSwitch(theme, defaultTheme)
            ]
          : [
              _buildSwitch(theme, defaultTheme),
              _buildTitle(theme, defaultTheme)
            ],
    );
  }
}
